#include "fractaldrawer.h"
#include <QLabel>
#include <QScrollArea>
#include <QMenu>
#include <QMenuBar>
#include <QStatusBar>
#include <QToolBar>
#include <QAction>
#include <QEventLoop>
#include <QMessageBox>
#include "dlgconfig.h"

FractalDrawer::FractalDrawer(QWidget *parent)
    : QMainWindow(parent), w(1680), h(1024),
    x1(-1.5), x2(1), y1(-1), y2(1), cRe(0), cIm(1), iters(128), exponent(2)
{
    setupForm();
    setupActions();
}

void FractalDrawer::setupForm(){
    lblImage = new QLabel;
    lblImage->setSizePolicy(QSizePolicy::Ignored,
                            QSizePolicy::Ignored);
    lblImage->setScaledContents(true);
    scrollArea = new QScrollArea;
    scrollArea->setWidget(lblImage);
    this->setCentralWidget(scrollArea);
    this->resize(w, h);
    this->setWindowTitle(tr("Fractal Drawer"));
}

void FractalDrawer::setupActions(){
    QIcon iconDraw(":/icons/draw.svg");
    actDraw = new QAction(tr("&Draw"));
    actDraw->setToolTip(tr("Draws fractal", "tool tip"));
    actDraw->setStatusTip(tr("Draws fractal", "status tip"));
    actDraw->setShortcut(tr("Ctrl+D", "shortcut for draw action"));
    actDraw->setIcon(iconDraw);


    QIcon iconAbort(":/icons/abort.svg");
    actAbort = new QAction(tr("&Abort"));
    actAbort->setEnabled(false);
    actAbort->setShortcut(QKeySequence::Cancel);
    actAbort->setToolTip(tr("Interrupts current drawing"));
    actAbort->setStatusTip(tr("Interrupts current drawing"));
    actAbort->setIcon(iconAbort);

    QIcon iconQuit(":/icons/quit.svg");
    actQuit = new QAction(tr("&Quit"));
    actQuit->setShortcut(QKeySequence::Quit);
    actQuit->setToolTip(tr("Exits the FractalViewer application"));
    actQuit->setStatusTip(tr("Exits the FractalViewer application"));
    actQuit->setIcon(iconQuit);

    QIcon iconPreferences = QIcon::fromTheme("preferences-system", QIcon(":/icons/preferences.svg"));
    actPreferences = new QAction(tr("&Preferences..."));
    actPreferences->setShortcut(QKeySequence::Preferences);
    actPreferences->setToolTip(tr("Opens dialog with preferences"));
    actPreferences->setStatusTip(tr("Opens dialog with preferences"));
    actPreferences->setIcon(iconPreferences);

    mnuFractal = new QMenu(tr("&Fractal"));
    mnuFractal->addAction(actDraw);
    mnuFractal->addAction(actAbort);
    mnuFractal->addSeparator();
    mnuFractal->addAction(actQuit);


    mnuTools = new QMenu(tr("&Tools"));
    mnuTools->addAction(actPreferences);

    toolBar = new QToolBar(tr("Main toolbar"));
    toolBar->addAction(actDraw);
    toolBar->addAction(actAbort);
    toolBar->addSeparator();
    toolBar->addAction(actQuit);
    toolBar->addSeparator();
    toolBar->addAction(actPreferences);

    this->addToolBar(toolBar);

    this->menuBar()->addMenu(mnuFractal);
    this->menuBar()->addMenu(mnuTools);

    this->setupConnections();
    this->statusBar()->showMessage(tr("Application is ready"), 2000);

}

void FractalDrawer::setupConnections(){
    connect(actDraw, SIGNAL(triggered()), this, SLOT(draw()));
    connect(actAbort, SIGNAL(triggered()), this, SLOT(abort()));
    connect(actQuit, SIGNAL(triggered()), this, SLOT(close()));
    connect(actPreferences, SIGNAL(triggered()), this, SLOT(configure()));
}

void FractalDrawer::configure(){
    DlgConfig config;
    config.setX1(x1);
    config.setX2(x2);
    config.setY1(y1);
    config.setY2(y2);
    config.setIterations(iters);
    config.setExponent(exponent);
    config.setRe(cRe);
    config.setIm(cIm);
    if(config.exec() == QDialog::Accepted){
        x1 = config.getX1();
        x2 = config.getX2();
        y1 = config.getY1();
        y2 = config.getY2();
        cRe = config.getRe();
        cIm = config.getIm();
        iters = config.getIterations();
        exponent = config.getExponent();
    }
}

void FractalDrawer::abort(){
    aborted = true;
}

void FractalDrawer::draw(){
    this->statusBar()->showMessage(tr("Drawing..."));
    QImage image(w, h, QImage::Format_Mono);
    int color;
    Real x, y;
    Real dx, dy; //width and height of pixel

    aborted = false;
    actAbort->setEnabled(true);
    actDraw->setEnabled(false);

    dx = (x2 - x1) / w;
    dy = (y2 - y1) / h;

    x = x1;
    QEventLoop loop;
    for(auto i = 0; i < w; i++){
        y = y1;
        for(auto j = 0; j < h; j++){
            color = mandelbrot(x, y, iters, exponent);
            image.setPixel(i, j, color);
            y = y + dy;
        }
        x = x + dx;
        loop.processEvents(QEventLoop::AllEvents);
        statusBar()->showMessage(tr("Drawing: %1 %").arg(i*100/w));
        if(aborted){
            break;
        }
    }
    if(aborted){
        statusBar()->showMessage(tr("Operation was aborted"), 2000);
    } else{
        lblImage->setPixmap(QPixmap::fromImage(image));
        lblImage->adjustSize();
        statusBar()->showMessage(tr("Operation completed"), 2000);
    }


    actAbort->setEnabled(false);
    actDraw->setEnabled(true);
}

FractalDrawer::~FractalDrawer()
{
}

